#!/bin/bash

SITLIST=$1 
OUTTYPE=$2
ERRORDIR=$3
source globalvar.sh
ymd2mjd() {
    local year=$1
    local mon=$((10#$2))
    local day=$((10#$3)) 
    [ $year -lt 100 ] && year=$((year + 2000))
    if [ $mon -le 2 ]; then
            mon=$(($mon + 12))
            year=$(($year - 1))
    fi
    local mjd=$(echo $year | awk '{print $1*365.25-$1*365.25%1-679006}')
    mjd=$(echo $mjd $year $mon $day | awk '{print $1+int(30.6001*($3+1))+2-int($2/100)+int($2/400)+$4}')
    echo $mjd
}
mjd2ydoy() {
    local mjd=$1
    local year=$((($mjd + 678940) / 365))
    local mjd0=$(ymd2mjd $year 1 1)
    local doy=$(($mjd - $mjd0))
    while [ $doy -le 0 ]; do
            year=$(($year - 1))
            mjd0=$(ymd2mjd $year 1 1)
            doy=$(($mjd - $mjd0 + 1))
    done
    printf "%d %03d\n" $year $doy
}
# func_SitnameNum2Char: used for changing the site name from number type to char type.
# usage: func_SitnameNum2Char sitname upper/lower
# exp: GD01 -> GDA1
func_SitnameNum2Char() {
    sitname_num="$1"
    model="$2"

    # change to upper
    sitname_num=$(echo "$sitname_num" | tr '[:lower:]' '[:upper:]')
    sitname_char="$sitname_num"

    # length
    if [ ${#sitname_num} -ne 4 ]; then
        if [ "$model" = "upper" ]; then
            echo "$sitname_char"
        else
            echo "$sitname_char" | tr '[:upper:]' '[:lower:]'
        fi
        return
    fi

    chars=($(echo "$sitname_num" | grep -o .))

    # dictionary
    declare -A digit_to_letter=(
        ['0']='A'
        ['1']='B'
        ['2']='C'
        ['3']='D'
        ['4']='E'
        ['5']='F'
        ['6']='G'
        ['7']='H'
        ['8']='I'
        ['9']='J'
    )

    # the last 2 chars is number?
    if ! [[ ${chars[2]} =~ ^[0-9]+$ ]] || ! [[ ${chars[3]} =~ ^[0-9]+$ ]]; then
        if [ "$model" = "upper" ]; then
            echo "$sitname_char"
        else
            echo "$sitname_char" | tr '[:upper:]' '[:lower:]'
        fi
        return
    fi

    # the second char is number?
    if ! [[ ${chars[1]} =~ ^[0-9]+$ ]]; then
        char_3rd_rp=${digit_to_letter[${chars[2]}]}
        if [ -n "$char_3rd_rp" ]; then
            chars[2]=$char_3rd_rp
        fi
    fi

    sitname_char=$(IFS=''; echo "${chars[*]}")
    if [ "$model" = "upper" ]; then
        echo "$sitname_char"
    else
        echo "$sitname_char" | tr '[:upper:]' '[:lower:]'
    fi
}
download_obs_15(){
    year=${1}
    yy=${year:2:2}
    doy=${2}
    starttime=${3}
    endtime=${4}  
    sitn1=${5}

	if [ ! -f ${OUTTYPE} ];then
		touch ${OUTTYPE}
	fi

    # if [ -f ${ERRORDIR}/csfile.txt ];then
        # rm ${ERRORDIR}/csfile.txt
    # fi
    # if [ ! -f ${ERRORDIR}/csfile.txt ];then
        # touch ${ERRORDIR}/csfile.txt
    # fi

    ExecuteWithoutOutput "python3 ${bindir}/py_getdwtime_form.py ${starttime} ${endtime} ${OUTTYPE}"  
    flag=0     
    flag1=0
    if [ -f $OUTTYPE ];then
        while read line
        do
            sitn_low=$(echo $line | tr  '[A-Z]' '[a-z]')    # 字母都小写, exp: b45 b 01 45
            sitntype=${sitn_low:0:3}                        # 具体小时分钟
            timetype=${sitn_low:4:1}                        # 具体小时
            hour=${sitn_low:6:2}                            # 小时
            minute=${sitn_low:9:2}                          # 分钟
            station=${sitn1}${doy}${sitntype}.${yy}b        # 数据文件全名，测站名大写
            # Alternative station
            sitn1_alter=$(func_SitnameNum2Char $sitn1 "upper")
            station_alter=${sitn1_alter}${doy}${sitntype}.${yy}b
            #http
            url="${http_obs}/${year}/${doy}/${yy}d/${timetype}/${minute}/${station}"
            url_alter="${http_obs}/${year}/${doy}/${yy}d/${timetype}/${minute}/${station_alter}"

            # echo "*********************************************************" >> "${ERRORDIR}/csfile.txt"
            # echo "${url}" >> "${ERRORDIR}/csfile.txt"
            
            #exit 1
            if [ -s ${ERRORDIR}/errorfile ];then
                rm ${ERRORDIR}/errorfile
            fi
            wget --spider $url > ${ERRORDIR}/errorfile 2>&1
            strflag=`grep "Remote file exists." ${ERRORDIR}/errorfile`

            # echo "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++" >> "${ERRORDIR}/csfile.txt"
            # echo $strflag >> "${ERRORDIR}/csfile.txt"

            if [ "$strflag" == "Remote file exists." ];then
                let flag=0
            else 
                let flag=1
            fi
            
            # if the sitn1 is not exist, we should check the sitn1_alter
            if [ ${flag} -eq 1 ];then
                rm ${ERRORDIR}/errorfile
                wget --spider $url_alter > ${ERRORDIR}/errorfile 2>&1
                strflag=`grep "Remote file exists." ${ERRORDIR}/errorfile`

                if [ "$strflag" == "Remote file exists." ];then
                    let flag=0
                else 
                    let flag=1
                fi
                echo "check alter ${sitn1} -> ${sitn1_alter}" >> "${ERRORDIR}/alterLog.txt"
            fi

            if [ ${flag} -eq 1 ];then
                let flag1=1
                echo $flag1
                #add save not have date station
                if [ -f ${ERRORDIR}/NODate.txt ];then
                    rm ${ERRORDIR}/NODate.txt
                fi
                if [ ! -f ${ERRORDIR}/NODate.txt ];then
		            touch ${ERRORDIR}/NODate.txt
	            fi
                echo ${sitn1} >> "${ERRORDIR}/NODate.txt"
                rm ${ERRORDIR}/errorfile
                break
            fi
            
            rm ${ERRORDIR}/errorfile

        done < $OUTTYPE
    fi
    if [ ${flag} -eq 0 -a ${flag1} -eq 0 ];then
        echo 0
    fi   
}
ExecuteWithoutOutput() {
    local cmd="$1"
    time=$(date +'%Y-%m-%d %H:%M:%S')
    $cmd >/dev/null 2>&1 
    if [ $? -eq 0 ]; then  
            return 0
            #echo "脚本实现"
    else
            echo "脚本未实现"
            return 1
    fi
}

if [ -f $SITLIST ];then   
    while read line
    do 
        sitn_low=$(echo $line | tr  '[A-Z]' '[a-z]')  
        sitn_upper=$(echo $line | tr '[a-z]' '[A-Z]')
        sitn1=`echo $sitn_upper | awk -F " " '{print $13}'`

        #按照结束时间读取YMD
        s_iyear=`echo $sitn_low | awk -F " " '{print $1}'`
        s_imonth=`echo $sitn_low | awk -F " " '{print $2}'`
        s_iday=`echo $sitn_low | awk -F " " '{print $3}'`
        s_ihour=`echo $sitn_low | awk -F " " '{print $4}'`
        s_iminute=`echo $sitn_low | awk -F " " '{print $5}'`
        s_isec=`echo $sitn_low | awk -F " " '{print $6}'`
        e_iyear=`echo $sitn_low | awk -F " " '{print $7}'`
        e_imonth=`echo $sitn_low | awk -F " " '{print $8}'`
        e_iday=`echo $sitn_low | awk -F " " '{print $9}'`
        e_ihour=`echo $sitn_low | awk -F " " '{print $10}'`
        e_iminute=`echo $sitn_low | awk -F " " '{print $11}'`
        e_isec=`echo $sitn_low | awk -F " " '{print $12}'`
        starttime="$s_iyear $s_imonth $s_iday $s_ihour $s_iminute $s_isec"
        endtime="$e_iyear $e_imonth $e_iday $e_ihour $e_iminute $e_isec"  #$OUTTYPY是类型输出位置
            
        #开始时间
        YMD1=${s_iyear}'-'${s_imonth}'-'${s_iday}   #解码需要用到的时间
        mjd1=$(ymd2mjd $s_iyear $s_imonth $s_iday)
        YDOY1=($(mjd2ydoy $mjd1))   
        doy1=${YDOY1[1]}      #下载数据、路劲需要
        #结束时间
        YMD2=${e_iyear}'-'${e_imonth}'-'${e_iday}   #解码需要用到的时间
        mjd2=$(ymd2mjd $e_iyear $e_imonth $e_iday)
        YDOY2=($(mjd2ydoy $mjd2))   
        doy2=${YDOY2[1]}      #下载数据、路劲需要
           
        if [ ${s_iday} -eq ${e_iday} ];then
            flag=`download_obs_15 ${s_iyear} ${doy1} "${starttime}" "${endtime}" ${sitn1}`
            #echo $flag
            if [ ${flag} -eq 1 ];then
                echo "This station ${sitn1} not have date!"
                rm $SITLIST
                #return 1
            fi
            if [ ${flag} -eq 0 ];then
                echo "This station ${sitn1} have date!"
            fi
        fi    
        if [ ${s_iday} -ne ${e_iday} ];then
            starttime1="$s_iyear $s_imonth $s_iday $s_ihour $s_iminute $s_isec"
            endtime1="$s_iyear $s_imonth $s_iday 24 00 00"
            starttime2="$e_iyear $e_imonth $e_iday 00 00 00"  #分为两个时段
            endtime2="$e_iyear $e_imonth $e_iday $e_ihour $e_iminute $e_isec"
            flag1=`download_obs_15 ${s_iyear} ${doy1} "${starttime1}" "${endtime1}" ${sitn1}`
            if [ ${flag1} -eq 1 ];then
                echo "This station ${sitn1} not have date!"
                rm $SITLIST
                #return 1
            fi
            if [ ${flag1} -eq 0 ];then
                echo "This station ${sitn1} have date!"
            fi
            flag2=`download_obs_15 ${e_iyear} ${doy2} "${starttime2}" "${endtime2}" ${sitn1}`
            if [ ${flag2} -eq 1 ];then
                echo "This station ${sitn1} not have date!"
                rm $SITLIST
                #return 1
            fi  
            if [ ${flag2} -eq 0 ];then
                echo "This station ${sitn1} have date!"
            fi
        fi    
    done < ${SITLIST}     
fi
