#!/usr/local/bin/python3

import sys
import math
import math as mt

a = 6378137.0  
b = 6356752.31414 
a = 6378137.0 
f = 1 / 298.257223563 
b = 6356752.31414  
sqrt = mt.sqrt
sin = mt.sin
cos = mt.cos
atan = mt.atan
atan2 = mt.atan2
fabs = math.fabs

def rad2angle(r):
    a = r * 180.0 / mt.pi
    return a

def angle2rad(a):
    r = a * mt.pi / 180.0
    return r

def XYZ2BLH(X, Y, Z):
    R = sqrt(X * X + Y * Y)
    B0 = atan2(Z, R)
    while 1:
        N = a / sqrt(1.0 - f * (2 - f) * sin(B0) * sin(B0))
        B_ = atan2(Z + N * f * (2 - f) * sin(B0), R)
        if (fabs(B_ - B0) < 1.0e-7):
            break
        B0 = B_
    L_ = atan2(Y, X)
    H = R / cos(B_) - N
    B = rad2angle(B_)
    L = rad2angle(L_)
    return B, L, H

def BLH2XYZ(B, L, H):
    B = angle2rad(B)  
    L = angle2rad(L)  
    e = sqrt((a ** 2 - b ** 2) / (a ** 2)) 
    N = a / sqrt(1 - e * e * sin(B) * sin(B)) 
    X = (N + H) * cos(B) * cos(L)
    Y = (N + H) * cos(B) * sin(L)
    Z = (N * (1 - e * e) + H) * sin(B)
    return X, Y, Z 

def XYZ2NEU(X, Y, Z, X1, Y1, Z1):
    dx = X - X1
    dy = Y - Y1
    dz = Z - Z1
    B, L, H = XYZ2BLH(X1, Y1, Z1)
    B = angle2rad(B)
    L = angle2rad(L)
    N = -sin(B) * cos(L) * dx - sin(B) * sin(L) * dy + cos(B) * dz
    E = -sin(L) * dx + cos(L) * dy
    U = cos(B) * cos(L) * dx + cos(B) * sin(L) * dy + sin(B) * dz
    H = sqrt(pow(N, 2) + pow(E, 2))  
    error = sqrt(pow(dx, 2) + pow(dy, 2) + pow(dz, 2))
    return H, U, error

def readbasestation(readpath):  #读取所有基准站点
    with open(readpath,'r',encoding='utf-8') as readfile:
        stationdata=[]
        for line in readfile:
            data_line=line.strip('\n').split() # 去除首尾换行符，并按空格划分 储存为列表
            data=[]
            data.append(data_line[0]) #name
            data.append(float(data_line[1])) #X
            data.append(float(data_line[2])) #Y
            data.append(float(data_line[3])) #Z
            stationdata.append(data)
        return stationdata
    pass

def readrequest(readpath):
     with open(readpath,'r',encoding='utf-8') as readfile:
        requestdata=[];num_row=0
        for line in readfile:
            if num_row>1:
                data_line=line.strip('\n').split() # 去除首尾换行符，并按空格划分 储存为列表
                reqdata=[]
                reqdata.append((data_line[15])) #id
                reqdata.append(float(data_line[1])) #lat
                reqdata.append(float(data_line[0])) #lon
                reqdata.append(float(data_line[2])) #height
                starttime=data_line[3]+' '+data_line[4]+' '+data_line[5]+' '+data_line[6]+' '+data_line[7]+' '+data_line[8]
                endtime=data_line[9]+' '+data_line[10]+' '+data_line[11]+' '+data_line[12]+' '+data_line[13]+' '+data_line[14]
                reqdata.append(starttime)   #start time
                reqdata.append(endtime)     #end time
                requestdata.append(reqdata)
            num_row=num_row+1
        return requestdata

def distance(point1,point2):
    dx=point1[0]-point2[0];dy=point1[1]-point2[1];dz=point1[2]-point2[2]
    return(mt.sqrt(dx*dx+dy*dy+dz*dz))
    pass
# def distance(point1,point2):
#     dx=point1[0]-point2[0];dy=point1[1]-point2[1]
#     return(mt.sqrt(dx*dx+dy*dy))
#     pass
def mindistance(basestation_data,request_one):  #传入的参数是浮点型的数字#返回最小距离索引
    index=0
    min_distance=9999999999
    request_xyz=BLH2XYZ(request_one[1],request_one[2],request_one[3])
    request_XYZ=[]
    for i in range(len(request_xyz)):request_XYZ.append(request_xyz[i])
    for i in range(len(basestation_data)):
        basestation_XYZ=[]
        for j in range(int(len(basestation_data[i]))-1):basestation_XYZ.append(basestation_data[i][j+1])
        if min_distance>distance(basestation_XYZ,request_XYZ):
            min_distance=distance(basestation_XYZ,request_XYZ)
            index=i
    return(index)
    pass

def removerepeatdata(data):  #移除列表中重复的数据(根据id判断)
    norepeatdata=[]
    if int(len(data))==1:
        norepeatdata.append(data[0])
        return norepeatdata
    for i in range(len(data)-1):
        if i==0:norepeatdata.append(data[i])
        if data[i][0] != data[i+1][0]:norepeatdata.append(data[i+1])#保留第一次出现的时刻
    return norepeatdata
    

if __name__  == '__main__':   #python3 ./mindisstation.py result/outstation.txt result/outrequest.txt result/usebasestation.txt
    message='命令错误'
    if len(sys.argv)<=1 :
        print(message)
        sys.exit()
    stationdata=readbasestation(sys.argv[1])
    requestdata=readrequest(sys.argv[2])
    #根据id保留格网点
    requestdata1=removerepeatdata(requestdata)
    #print(requestdata1)
    #找出每个请求对应最近的三个站点（其实一个就够，主要是备用）
    allmin_id=[]
    for i in range(len(requestdata1)):
        min_id=[]
        copystation=[]
        for k in range(len(stationdata)):
            copystation.append(stationdata[k])
        for j in range(int(sys.argv[4])):
            id_one=mindistance(copystation,requestdata1[i])
            min_id.append(copystation[id_one][0]) #存储测站名
            del copystation[int(id_one)]
        allmin_id.append(min_id)
        #print(min_id)
    #写出最终使用基准站的文档
    with open(sys.argv[3],'w',encoding='utf-8') as writefile:
        writefile.write('Grid                                     start_time          end_time            min_d_base'+'\n')
        for i in range(len(allmin_id)):
            str1=str((requestdata1[i][0]))+" "+str('{:.8f}'.format(float(requestdata1[i][1])))+" "+str('{:.8f}'.format(float(requestdata1[i][2])))+" "+str('{:.8f}'.format(float(requestdata1[i][3])))+" "+requestdata1[i][4]+" "+requestdata1[i][5]+" "
            str2=""
            for j in range(int(sys.argv[4])):
                str2=str2+allmin_id[i][j]+" "
            str2=str2+'\n'
            str3=str1+str2
            writefile.write(str3)
        pass
                
    
