# coding=utf-8
import os
import copy
import math
from math import nan,isnan
import sys

class sitatom(object):
    def __init__(self):
        self.sitname_base=""
        self.sitname_rove=""

        self.satlist_GPS = list()
        self.satlist_remove = []
        self.satlist_GPS = ['G01','G02','G03','G04','G05','G06','G07','G08','G09','G10',
            'G11','G12','G13','G14','G15','G16','G17','G18','G19','G20','G21',
            'G22','G23','G24','G25','G26','G27','G28','G29','G30','G31','G32'
            ]
        self.satlist_BD2 = list()
        self.satlist_BD2 = ['C01','C02','C03','C04','C05','C06','C07','C08','C09','C10',
            'C11','C12','C13','C14','C15','C16'
            ]
        self.satlist_BD3 = list()
        self.satlist_BD3 = ['C19','C20','C21','C22','C23','C24','C25','C26','C27','C28',
            'C29','C30','C32','C33','C34','C35','C36','C37','C38','C39','C40',
            'C41','C42','C43','C44','C45','C46',
            ]
        self.satlist_GAL = list()
        self.satlist_GAL = ['E01','E02','E03','E04','E05','E06','E07','E08','E09','E10',
            'E11','E12','E13','E14','E15','E18','E19','E20','E21','E22','E23',
            'E24','E25','E26','E27','E28','E29','E30','E31','E32','E33','E34',
            'E35','E36'
            ]
        self.satlist_QZS = list()
        self.satlist_QZS = ['J01','J02','J03','J04','J07']
        self.satlist_GLO = list()
        self.satlist_GLO = ['R01','R02','R03','R04','R05','R06','R07','R08','R09','R10',
            'R11','R12','R13','R14','R15','R16','R17','R18','R19','R20','R21',
            'R22','R23','R24','R25','R26','R27'
            ]

        # 卫星列表
        self.satlist = self.satlist_GPS + self.satlist_QZS + self.satlist_GLO + self.satlist_BD2 + self.satlist_BD3 + self.satlist_GAL
        # 卫星个数
        self.nsat = len(self.satlist)
        # 系统个数
        self.nsys = 4
        self.csys = ["GPS","BDS-2","BDS-3","Galileo","QZSS","GLONASS"]

        self.smjd=nan
        self.ssod=nan
        self.emjd=nan
        self.esod=nan
        self.smjdsod=nan
        self.emjdsod=nan
        self.intv = 0.00
        self.nepoch = 0

        # 1: ION, 2: ZTD
        self.atomtype=0
        # 测站单差值
        self.sdatomlist = []
        # 测站高度角
        self.sdelevlist = []
        self.sdazimlist = []
        # 双差参考星
        self.ddrsatlist = []
        # 测站双差值
        self.ddatomlist = []
        # amb状态
        self.ambtaglist = []

    def initdata(self,mjd0,sod0,mjd1,sod1,intv,type):
        self.sitname_base=""
        self.sitname_rove=""
        self.smjd=nan
        self.ssod=nan
        self.emjd=nan
        self.esod=nan
        self.smjdsod=nan
        self.emjdsod=nan
        self.intv = 0.00
        self.nepoch = 0
        self.atomtype = 0
        self.sdatomlist = []
        self.sdelevlist = []
        self.sdazimlist = []
        self.ddrsatlist = []
        self.ddatomlist = []
        self.ambtaglist=[]
        t_nepoch = round(((mjd1-mjd0)*86400.0+(sod1-sod0))/intv)+1
        if t_nepoch > 0 and (type == 1 or type == 2):
            self.smjd=mjd0
            self.ssod=sod0
            self.emjd=mjd1
            self.esod=sod1
            self.smjdsod=mjd0+sod0/86400.0
            self.emjdsod=mjd1+sod1/86400.0
            self.intv = intv
            self.nepoch = t_nepoch
            self.atomtype=type
            self.sdatomlist = [[nan for col in range(self.nepoch)] for row in range(self.nsat)]
            self.sdelevlist = [[nan for col in range(self.nepoch)] for row in range(self.nsat)]
            self.sdazimlist = [[nan for col in range(self.nepoch)] for row in range(self.nsat)]
            self.ddrsatlist = [[nan for col in range(self.nepoch)] for row in range(self.nsat)]
            self.ddatomlist = [[nan for col in range(self.nepoch)] for row in range(self.nsat)]
            self.ambtaglist = [[nan for col in range(self.nepoch)] for row in range(self.nsat)]

def index_sys(cprn):
    csys = cprn[0:1]
    isys = nan
    satlist_BD2 = ['C01','C02','C03','C04','C05','C06','C07','C08','C09','C10',
        'C11','C12','C13','C14','C15','C16'
        ]
    satlist_BD3 = ['C19','C20','C21','C22','C23','C24','C25','C26','C27','C28',
        'C29','C30','C32','C33','C34','C35','C36','C37','C38','C39','C40',
        'C41','C42','C43','C44','C45','C46',
        ]

    if csys == "G":
        isys = 0
    elif csys == "C":
        if cprn in satlist_BD2:
            isys = 1
        elif cprn in satlist_BD3:
            isys = 2
    elif csys == "E":
        isys = 3
    elif csys == "J":
        isys = 4
    elif csys == "R":
        isys = 5
    return isys

def read_sdatomfile(intv, atomfile, sitatom, type=1):
    """
    type=1: extract from residual file, the value is [7]
                                                                   ION/ZTD
        ION CQ09 E36   59863   14400.00   55.010000  158.879900    1.233010
        0   1    2     3       4          5          6             7
    type=2: generate by bamboo, the value is [6]
                                                       ION/ZTD
        ION VR01 C26   59863   34170.00    0.000000    2.544381 CQ19
        0   1    2     3       4           5           6        7
    """
    if intv <= 0:
        return 0
    if not (type == 1 or type == 2):
        return 0
    mjd0=nan
    sod0=nan
    mjd1=nan
    sod1=nan
    # 1: ION, 2:ZTD
    atomtype=0
    try:
        # get the atom type, and initial the class
        fid = open(atomfile)
        for line in fid:
            if len(line.strip()) <= 0:
                continue
            value = line.split()

            # get the atom type
            if atomtype == 0:
                if value[0] == "ION":
                    atomtype=1
                elif value[0] == "ZTD":
                    atomtype=2
            
            # get the period of time
            mjd=float(value[3])
            sod=float(value[4])

            if isnan(mjd0) or isnan(sod0) or isnan(mjd1) or isnan(sod1):
                mjd0 = mjd
                sod0 = sod
                mjd1 = mjd
                sod1 = sod
            
            if (mjd0+sod0/86400.0) >= (mjd+sod/86400.0):
                mjd0 = mjd
                sod0 = sod
            if (mjd1+sod1/86400.0) <= (mjd+sod/86400.0):
                mjd1 = mjd
                sod1 = sod

        if atomtype == 0:
            return 0

        if isnan(mjd0) or isnan(sod0) or isnan(mjd1) or isnan(sod1):
            return 0

        nepoch = round(((mjd1-mjd0)*86400.0+(sod1-sod0))/intv)+1
        if nepoch <= 0:
            return 0
        
        sitatom.initdata(mjd0,sod0,mjd1,sod1,intv,atomtype)

        fid.close()
    except:
        print("***ERROR(tool_qcForAtom.py): some error in read_ambfile().")
        return 0

    try:
        fid = open(atomfile)
        for line in fid:
            if len(line.strip()) <= 0:
                continue
            value = line.split()
            t_base = value[1]
            t_rove = ""
            t_prn = value[2]
            t_mjd = float(value[3])
            t_sod = float(value[4])
            t_elev = float(value[5])
            t_atom = nan
            t_azim = nan
            if type == 1:
                t_atom = float(value[7])
                t_azim = float(value[6])
            elif type == 2:
                t_atom = float(value[6])
                t_rove = value[7]
            t_mjdsod = t_mjd+t_sod/86400.0
            if t_mjdsod < sitatom.smjdsod or t_mjdsod > sitatom.emjdsod:
                continue
            if not t_prn.strip() in sitatom.satlist:
                continue
            isat = sitatom.satlist.index(t_prn.strip())
            iepoch = round((t_mjdsod - sitatom.smjdsod)*86400.0/sitatom.intv)
            if atomtype == 1 or atomtype == 2:
                sitatom.sdatomlist[isat][iepoch] = t_atom
                sitatom.sdelevlist[isat][iepoch] = t_elev
                sitatom.sdazimlist[isat][iepoch] = t_azim
            if len(sitatom.sitname_base) == 0 and len(t_base) > 0:
                sitatom.sitname_base = t_base
            if len(sitatom.sitname_rove) == 0 and len(t_rove) > 0:
                sitatom.sitname_rove = t_rove

        fid.close()
    except:
        print("***ERROR(tool_qcForAtom.py): some error in read_atomfile().")
        return 0

    return 1

def read_ambfile(sitatom, ambfile):
    if sitatom.nepoch <= 0:
        print("***ERROR(tool_qcForAtom.py): there is no data in ion/ztd file.")
        return 0
    try:
        # get the atom type, and initial the class
        fid = open(ambfile)
        for line in fid:
            if len(line.strip()) <= 0:
                continue
            if ("[now]" in line and "[t0]" in line):
                # SH01 SH04 C37 59870   7480.0 [now] 59870   7340.0 [t0] -001 -007 --> 0003 -002     22.05 (elev)
                value = line.split()
                if len(value) < 16:
                    continue
                t_prn = value[2]
                t_smjd = float(value[6])
                t_ssod = float(value[7])
                t_emjd = float(value[3])
                t_esod = float(value[4])

                t_smjdsod = t_smjd + t_ssod/86400.0
                t_emjdsod = t_emjd + t_esod/86400.0
                if t_smjdsod >= t_emjdsod:
                    continue
                if t_smjdsod < sitatom.smjdsod or t_smjdsod > sitatom.emjdsod:
                    continue
                if t_emjdsod < sitatom.smjdsod or t_emjdsod > sitatom.emjdsod:
                    continue
                if not t_prn.strip() in sitatom.satlist:
                    continue
                t_sepoch = round((t_smjdsod - sitatom.smjdsod)*86400.0/sitatom.intv)
                t_eepoch = round((t_emjdsod - sitatom.smjdsod)*86400.0/sitatom.intv)

                isat = sitatom.satlist.index(t_prn.strip())

                for iepoch in range(t_sepoch, t_eepoch):
                    sitatom.ambtaglist[isat][iepoch] = 1
            
        fid.close()
    except:
        print("***ERROR(tool_qcForAtom.py): some error in read_ambfile().")
        return 0
    return 1

def detect_slip(sitatom):
    slip_limit=0.15 # unit: m
    if sitatom.nepoch <= 1:
        print("***ERROR(tool_qcForAtom.py): there is no data in ion/ztd file.")
        return 0
    for isat in range(sitatom.nsat):
        # opposite direction
        for iepoch in range(sitatom.nepoch-1,-1,-1):
            # check iepoch
            if iepoch <= 0 or iepoch >= sitatom.nepoch:
                continue
            # check data
            if isnan(sitatom.sdatomlist[isat][iepoch]) or isnan(sitatom.sdatomlist[isat][iepoch-1]):
                continue
            # confirm the reference satellite.
            rsat = nan
            elev_max = -1
            for i in range(sitatom.nsat):
                if isat == i:
                    continue
                if isnan(sitatom.sdatomlist[i][iepoch]) or isnan(sitatom.sdatomlist[i][iepoch-1]):
                    continue
                if sitatom.sdelevlist[i][iepoch] > elev_max:
                    elev_max = sitatom.sdelevlist[i][iepoch]
                    rsat = i
            if isnan(rsat):
                continue

            # detect slip
            t=(sitatom.sdatomlist[isat][iepoch]-sitatom.sdatomlist[isat][iepoch-1])-(sitatom.sdatomlist[rsat][iepoch]-sitatom.sdatomlist[rsat][iepoch-1])
            if abs(t)>slip_limit:
                print('%s%8d' %(sitatom.satlist[isat],iepoch))
                pass
    return 1

def write_atom(sitatom,filename):
    if sitatom.nepoch <= 0:
        print("***ERROR(tool_qcForAtom.py): there is no data in ion/ztd file.")
        return 0
    fid_w = open(filename,"w")
    for iepoch in range(sitatom.nepoch):
        for isat in range(sitatom.nsat):
            tmp_base = sitatom.sitname_base
            tmp_prn = sitatom.satlist[isat]
            tmp_mjdsod = sitatom.smjdsod+iepoch*sitatom.intv/86400.0
            tmp_mjd = math.floor(tmp_mjdsod)
            tmp_sod = (tmp_mjdsod-tmp_mjd)*86400.0
            tmp_atom = sitatom.sdatomlist[isat][iepoch]
            tmp_elev = sitatom.sdelevlist[isat][iepoch]
            tmp_azim = sitatom.sdazimlist[isat][iepoch]
            tmp_ambtag = sitatom.ambtaglist[isat][iepoch]

            if isnan(tmp_elev):
                tmp_elev=0.0
            if isnan(tmp_azim):
                tmp_azim=0.0
            if isnan(tmp_atom):
                continue
            if not isnan(tmp_ambtag):
                # ambtag: 1: amb is slip, need to delete
                if tmp_ambtag == 1:
                    continue
            if sitatom.atomtype == 1:
                fid_w.write("ION %4s %3s   %5d   %8.2f%12.6f%12.6f%12.6f\n" % (tmp_base.upper(), tmp_prn.upper(), tmp_mjd, tmp_sod, tmp_elev, tmp_azim, tmp_atom))
            elif sitatom.atomtype == 2:
                fid_w.write("ZTD %4s %3s   %5d   %8.2f%12.6f%12.6f%12.6f\n" % (tmp_base.upper(), tmp_prn.upper(), tmp_mjd, tmp_sod, tmp_elev, tmp_azim, tmp_atom))
    fid_w.close()
    return 1

def main(intv,type,atomfile,ambfile):
    # check input
    ## interval
    try:
        intv=float(intv)
    except:
        print("***ERROR(tool_qcForAtom.py): the input interval maybe wrong: ",intv)
        return
    if intv <= 0:
        print("***ERROR(tool_qcForAtom.py): the input interval maybe wrong: ",intv)
        return
    ## atom format type
    try:
        type=int(type)
    except:
        print("***ERROR(tool_qcForAtom.py): the input type of atomfile format maybe wrong: ",type)
        return
    if not (type == 1 or type ==2):
        print("***ERROR(tool_qcForAtom.py): the input type of atomfile format maybe wrong: ",type)
        return
    ## atom file
    if not os.path.exists(atomfile):
        print("***ERROR(tool_qcForAtom.py): Not find %s" %(atomfile))
        return
    if not os.path.exists(ambfile):
        print("***ERROR(tool_qcForAtom.py): Not find %s" %(ambfile))
        return
    sit = sitatom()
    if not read_sdatomfile(intv,atomfile,sit,type):
        return
    if not read_ambfile(sit, ambfile):
        return
    if not write_atom(sit, atomfile+"_qc"):
        return

if __name__ == "__main__":
    # intv="5"
    # type="1" # ionamb/ztdamb: 1; ion_interp/ztd_intep: 2
    # atomfile="C:\\Userdata\\Document\\Job\\Kepler\\project\\2023-02-BaiduVRS\\ion\\ionambcon_2022291_sh01-sh04"
    # ambfile="C:\\Userdata\\Document\\Job\\Kepler\\project\\2023-02-BaiduVRS\\ion\\log"
    # main(intv,type,atomfile,ambfile)

    message='Usage: tool_qcForAtom.py intv type ion/ztdfile amblogfile'
    if not len(sys.argv)==5:
        print(message)
        sys.exit()

    main(sys.argv[1],sys.argv[2],sys.argv[3],sys.argv[4])
