#!/usr/local/bin/python3

import numpy as np
import sys
import math as mt

a = 6378137.0  
b = 6356752.31414 
a = 6378137.0 
f = 1 / 298.257223563 
b = 6356752.31414  
sqrt = mt.sqrt
sin = mt.sin
cos = mt.cos
atan = mt.atan
atan2 = mt.atan2
fabs = mt.fabs

def rad2angle(r):
    a = r * 180.0 / mt.pi
    return a

def angle2rad(a):
    r = a * mt.pi / 180.0
    return r

def XYZ2BLH(X, Y, Z):
    R = sqrt(X * X + Y * Y)
    B0 = atan2(Z, R)
    while 1:
        N = a / sqrt(1.0 - f * (2 - f) * sin(B0) * sin(B0))
        B_ = atan2(Z + N * f * (2 - f) * sin(B0), R)
        if (fabs(B_ - B0) < 1.0e-7):
            break
        B0 = B_
    L_ = atan2(Y, X)
    H = R / cos(B_) - N
    B = rad2angle(B_)
    L = rad2angle(L_)
    return B, L, H

def BLH2XYZ(B, L, H):
    B = angle2rad(B)  
    L = angle2rad(L)  
    e = sqrt((a ** 2 - b ** 2) / (a ** 2)) 
    N = a / sqrt(1 - e * e * sin(B) * sin(B)) 
    X = (N + H) * cos(B) * cos(L)
    Y = (N + H) * cos(B) * sin(L)
    Z = (N * (1 - e * e) + H) * sin(B)
    return X, Y, Z 
def readGrid(filepath):
    with open(filepath,'r',encoding='utf-8') as readfile:
        XYZ=[]
        num=0
        for line in readfile:
            BLH=[]
            if num>0:
                data_line=line.strip('\n').split()
                BLH.append(float(data_line[1]));BLH.append(float(data_line[2]));BLH.append(float(data_line[3]))
                xyz=BLH2XYZ(BLH[0],BLH[1],BLH[2])
                XYZ.append(xyz)
            num=num+1
        return XYZ
def readsame(filepath):
    with open(filepath,'r',encoding='utf-8') as readfile:
        sameform=""
        num=0
        for line in readfile:
            if num>0:
                data_line=line.strip('\n').split()
                sameform=data_line[0]+" "+data_line[1]+" "+data_line[2]+" "+data_line[3]+" "+data_line[4]+" "+data_line[5]+" "+data_line[6]+" "+data_line[7]+" "+data_line[8]+" "+data_line[9]+" "+data_line[10]+" "+data_line[11]+" "+data_line[12]+" "+data_line[13]+" "+data_line[14]+" "+data_line[15]
            num=num+1
        return sameform
def readstationXYZ(filepath):
    with open(filepath,'r',encoding='utf-8') as readfile:
        Station=[]
        for line in readfile:
            station=[]
            data_line=line.strip('\n').split()
            station.append(data_line[0])#name
            station.append(float(data_line[1]))#X
            station.append(float(data_line[2]))#Y
            station.append(float(data_line[3]))#Z
            Station.append(station)
        return Station 
def calculatedis(point1,point2):
    dx=float(point1[0])-float(point2[0])
    dy=float(point1[1])-float(point2[1])
    dz=float(point1[2])-float(point2[2])
    return sqrt(dx*dx+dy*dy+dz*dz)
               
def findminsta(Grid,Station):
    mindis=999999999
    min_id=3
    grid=Grid[0]
    for i in range(len(Station)):
        point=[]
        point.append(Station[i][1]);point.append(Station[i][2]);point.append(Station[i][3])
        dis=calculatedis(grid,point)
        if dis<mindis:mindis=dis;min_id=i
    return min_id
        
        

if __name__  == '__main__':   
    message='命令错误'
    if len(sys.argv)<=1 :
        print(message)
        sys.exit()
    Grid=readGrid(sys.argv[1])#格网点的XYZ,以元组存储
    #print(Grid)
    Station=readstationXYZ(sys.argv[2])
    #print(Station)
    min_id=findminsta(Grid,Station)
    #print(min_id)
    minsta_name=Station[min_id][0]
    #print(minsta_name)
    #写新文件
    sameform=readsame(sys.argv[1])
    #print(sameform)
    with open(sys.argv[3],'w',encoding='utf-8') as writefile:
        writefile.write('Grid                                     start_time          end_time            min_d_base'+'\n')
        strform=sameform+" "+minsta_name+"\n"
        writefile.write(strform)
    
                
    
    