#!/bin/bash
# PROCESS CONFIGURE FOR RTK SMOOTH. CONFIGURE FORMAT
#### Configure File:
# TIME:  2021 7 16 0 0 0 86400
# OBSDIR:  /home/kpl/xps/
# BRDMDIR: /home/kpl/xps/brdm/
# NSIT: 6
# RECORD1: HB01 HB02
# RECORD2: HB02 HB03

#### XYZ File:
# HB01 x y z
# HB02 x y z
# HB03 x y z
main() {
    if [ $# -ne 2 ]; then
        printUsage
        exit 1
    fi
    if [ ! -f tiny.conf ];then
        echo "tiny.conf not exsit!"
        exit 1
    fi
    conf=$1
    xyzfile=$2

    # grep time
    iy=$(awk '{if($1~/^TIME/) print $2}' $conf)
    im=$(awk '{if($1~/^TIME/) print $3}' $conf)
    id=$(awk '{if($1~/^TIME/) print $4}' $conf)
    ih=$(awk '{if($1~/^TIME/) print $5}' $conf)
    ii=$(awk '{if($1~/^TIME/) print $6}' $conf)
    is=$(awk '{if($1~/^TIME/) print $7}' $conf)
    ss=$(awk '{if($1~/^TIME/) print $8}' $conf)
    sed -i 's/pos-time=.*/pos-time='${iy}' '${im}' '${id}' '${ih}' '${ii}' '${is}' '${ss}'/g' tiny.conf
    yy=${iy:2:2}
    smjd=$(ymd2mjd ${iy} ${im} ${id})
    sydoy=($(mjd2ydoy $smjd))
    echo "sody: ${sydoy[0]} ${sydoy[1]}"

    # get end day  #出现09会有bug
    isod=$(( 10#${ih}*3600+10#${im}*60+10#${is} ))
    dd=$(( (${isod}+${ss})/86401 ))
    emjd=$(( ${smjd}+${dd} ))
    eydoy=($(mjd2ydoy $emjd))
    echo "edoy: ${eydoy[0]} ${eydoy[1]}"

    # grep obsdir and brdmdir
    obsdir=$(awk '{if($1~/^obsdir/) print $2}' $conf)
    brdmdir=$(awk '{if($1~/^brdmdir/) print $2}' $conf)

    # grep nsit
    nbaseline=$(awk '{if($1~/^nbaseline/) print $2}' $conf)
    echo $nbaseline
    ## iterate each baseline
    for ((i=1;i<=${nbaseline};i++));do
        rove=$(awk '{if($1~/^RECORD'${i}':/) print $2}' $conf)
        base=$(awk '{if($1~/^RECORD'${i}':/) print $3}' $conf)
        echo $rove $base
        prepareConfigures $rove $base $xyzfile
        rove=$(echo $rove | tr 'A-Z' 'a-z')
        base=$(echo $base | tr 'A-Z' 'a-z')
        

        ###ppp 
        # sed -i 's/pos-posmode =.*/pos-posmode = ppp-kinematic/g' tiny.conf
        # sed -i 's/pos-baseline=.*/pos-baseline='${base}' '${rove}'/g' tiny.conf
        # ./rtk "-k" "tiny.conf" "${obsdir}/2021/197/${rove}${ydoy[1]}0.${yy}o" "${obsdir}/whu%W%D.clk" "${brdmdir}/whu%W%D.sp3"

        ##########################################################################################################################################################
        # zgj/jyh smooth and iar here
        # TODO:need to specify ${infil} and ${outpath} in this shell beforehand, and meanwhile set the EWL/WL/N1 constrain file directory in tiny.conf.
        sed -i 's/pos-smooth=.*/pos-smooth=0/g' tiny.conf
        # rtk
        sed -i 's/pos-posmode=.*/pos-posmode=rtk-kinematic/g' tiny.conf
        sed -i 's/stats-stdpos=.*/stats-stdpos=5000/g' tiny.conf

        flist=""
        for ((doy=${sydoy[1]};doy<=${eydoy[1]};doy++))
        do
            flist="${flist} ${obsdir}/${iy}/${doy}/${base}${doy}0.${yy}o ${obsdir}/${iy}/${doy}/${rove}${doy}0.${yy}o ${brdmdir}/brdm${doy}0.${yy}p"
        done
        echo "file_list: ${flist}"

        ./rtk -k tiny.conf -ti 5 ${flist} 2>&1 | tee result.log  
        result=$(tail -n 1 result.log)  
        #echo $result  
        new_x=`echo $result | awk -F " " '{print $3}'`
        new_y=`echo $result | awk -F " " '{print $4}'`
        new_z=`echo $result | awk -F " " '{print $5}'`
        dx=`echo $result | awk -F " " '{print $6}'`
        dy=`echo $result | awk -F " " '{print $7}'`
        dz=`echo $result | awk -F " " '{print $8}'`
        dx=`echo $dx | awk '{print (1000*sqrt($1*$1))}'`
        dy=`echo $dy | awk '{print (1000*sqrt($1*$1))}'`
        dz=`echo $dz | awk '{print (1000*sqrt($1*$1))}'` 
        #echo $dx $dy $dz
        if [ $dx -gt 50 -o $dy -gt 50 -o $dz -gt 50 ];then
            #echo "yes"
            #需要更新坐标重新计算
            sed -i 's/ant-pos1=.*/ant-pos1='$new_x'/g' tiny.conf
            sed -i 's/ant-pos2=.*/ant-pos2='$new_y'/g' tiny.conf
            sed -i 's/ant-pos3=.*/ant-pos3='$new_z'/g' tiny.conf
            #改为静态，不约束坐标计算
            sed -i 's/pos-posmode=.*/pos-posmode=rtk-static/g' tiny.conf
            ./rtk -k tiny.conf -ti 5 ${flist} 2>&1 | tee result.log  
            result=$(tail -n 1 result.log)  
            #echo $result  
            new_x=`echo $result | awk -F " " '{print $3}'`
            new_y=`echo $result | awk -F " " '{print $4}'`
            new_z=`echo $result | awk -F " " '{print $5}'`
            dx=`echo $result | awk -F " " '{print $6}'`
            dy=`echo $result | awk -F " " '{print $7}'`
            dz=`echo $result | awk -F " " '{print $8}'`
            dx=`echo $dx | awk '{print (1000*sqrt($1*$1))}'`
            dy=`echo $dy | awk '{print (1000*sqrt($1*$1))}'`
            dz=`echo $dz | awk '{print (1000*sqrt($1*$1))}'`
            if [ $dx -lt 50 -a $dy -lt 50 -a $dz -lt 50 ];then
                sed -i 's/ant-pos1=.*/ant-pos1='$new_x'/g' tiny.conf
                sed -i 's/ant-pos2=.*/ant-pos2='$new_y'/g' tiny.conf
                sed -i 's/ant-pos3=.*/ant-pos3='$new_z'/g' tiny.conf
                sed -i 's/stats-stdpos=.*/stats-stdpos=0.05/g' tiny.conf
                ./rtk -k tiny.conf -ti 5 ${flist} 2>&1 | tee result.log  
                result=$(tail -n 1 result.log)  
                #echo $result  
                new_x=`echo $result | awk -F " " '{print $3}'`
                new_y=`echo $result | awk -F " " '{print $4}'`
                new_z=`echo $result | awk -F " " '{print $5}'`
                #更新xyz文件
                #str=$(grep ${rove} $xyzfile) 
                pos1=$(awk '{if($1~/^'${rover}'/) print $2}' $xyzfile)
                pos2=$(awk '{if($1~/^'${rover}'/) print $3}' $xyzfile)
                pos3=$(awk '{if($1~/^'${rover}'/) print $4}' $xyzfile)
                str_old="${rove} $pos1 $pos2 $pos3"
                str_old=$(echo $str_old | tr 'a-z' 'A-Z')
                str_new="${rove} ${new_x} ${new_y} ${new_z}"
                str_new=$(echo $str_new | tr 'a-z' 'A-Z')
                rove=$(echo $rove | tr 'a-z' 'A-Z')
                #echo $new_x $new_y $new_z
                num=$(grep -n "${rove}" $xyzfile | awk -F ":" '{print $1}')
                #num=$(sed -n "/${rove}/=" $xyzfile)
                echo $num
                sed -i "${num}d" $xyzfile
                echo $str_new >> $xyzfile
                #sed -i "/${str_old}/c$str_new/" $xyzfile
                #sed -i "s/${rove}*/${rove} ${new_x} ${new_y} ${new_z}/g" $xyzfile
                echo "The XYZ of ${rove} is update success!"
            else
                echo "The XYZ of ${rove} is update fail!"
            fi
        else
            echo "The XYZ of ${rove} is right!"
        fi
        
        #echo "###MESSAGE(rtk_smooth.sh): Processing the NO.${i} finished"
        #恢复默认设置
        sed -i 's/pos-smooth=.*/pos-smooth=1/g' tiny.conf
        sed -i 's/pos-posmode=.*/pos-posmode=rtk-static/g' tiny.conf
        sed -i 's/stats-stdpos=.*/stats-stdpos=0.05/g' tiny.conf
    done
}
prepareConfigures() {
    # get data
    local rover=$1
    local base=$2
    local xyzfile=$3

    # read data
    # a=$(cat -n test.conf|grep pos-time=|awk '{print $1}')
    pos1=$(awk '{if($1~/^'${rover}'/) print $2}' $xyzfile)
    pos2=$(awk '{if($1~/^'${rover}'/) print $3}' $xyzfile)
    pos3=$(awk '{if($1~/^'${rover}'/) print $4}' $xyzfile)
    ref1=$(awk '{if($1~/^'${base}'/) print $2}' $xyzfile)
    ref2=$(awk '{if($1~/^'${base}'/) print $3}' $xyzfile)
    ref3=$(awk '{if($1~/^'${base}'/) print $4}' $xyzfile)

    # trans A to a
    rover=$(echo $rover | tr 'A-Z' 'a-z')
    base=$(echo $base | tr 'A-Z' 'a-z')

    # modify config file coord
    sed -i 's/pos-baseline=.*/pos-baseline='${rover}' '${base}'/g' tiny.conf    
    sed -i 's/ant-pos1=.*/ant-pos1='$pos1'/g' tiny.conf
    sed -i 's/ant-pos2=.*/ant-pos2='$pos2'/g' tiny.conf
    sed -i 's/ant-pos3=.*/ant-pos3='$pos3'/g' tiny.conf
    sed -i 's/ant-ref1=.*/ant-ref1='$ref1'/g' tiny.conf
    sed -i 's/ant-ref2=.*/ant-ref2='$ref2'/g' tiny.conf
    sed -i 's/ant-ref3=.*/ant-ref3='$ref3'/g' tiny.conf

    # create tempdir
    sed -i 's/file-tempdir=.*/file-tempdir='temp_${rover}_${base}'/g' tiny.conf
}
mjd2ydoy() {
    local mjd=$1
    local year=$((($mjd + 678940) / 365))
    local mjd0=$(ymd2mjd $year 1 1)
    local doy=$(($mjd - $mjd0))
    while [ $doy -le 0 ]; do
        year=$(($year - 1))
        mjd0=$(ymd2mjd $year 1 1)
        doy=$(($mjd - $mjd0 + 1))
    done
    printf "%d %03d\n" $year $doy
}
ymd2mjd() {
    local year=$1
    local mon=$((10#$2))
    local day=$((10#$3))
    [ $year -lt 100 ] && year=$((year + 2000))
    if [ $mon -le 2 ]; then
        mon=$(($mon + 12))
        year=$(($year - 1))
    fi
    local mjd=$(echo $year | awk '{print $1*365.25-$1*365.25%1-679006}')
    mjd=$(echo $mjd $year $mon $day | awk '{print $1+int(30.6001*($3+1))+2-int($2/100)+int($2/400)+$4}')
    #local mjd=$(bc <<< "$year*365.25 - $year*365.25 % 1 - 679006")
    #mjd=$(bc <<< "($mjd + (30.6001*($mon+1))/1 + 2 - $year/100 + $year/400 + $day)/1")
    echo $mjd
}
printUsage() {
    echo "-----------------------------------------------------"
    echo " GNSS POST-PROCESS RTK SCRIPT FOR TINYPPP PROGRAM"
    echo " Usage: rtk_smooth.sh conf xyzfile BINDIR SYS"
    echo "    ex: rtk_smooth.sh conf xyzfile bindir GRECJ"
    echo "-----------------------------------------------------"
}
ExecuteWithoutOutput() {
    local cmd="$1"
    time=$(date +'%Y-%m-%d %H:%M:%S')
    $cmd >/dev/null 2>&1 
    if [ $? -eq 0 ]; then  
        #echo "脚本实现"
        return 0
    else
        echo "Execute ERROR!"
        return 1
    fi
}
main "$@"
