#!/bin/bash

if [ $# -ne 1 ]; then
	echo "install.sh INSTALL_DIR"
	exit 1
fi
INSTALL_DIR=$1
builddir() {
	local dirpath=$1
	if [ ! -d ${dirpath} ]; then
		mkdir -p ${dirpath}
	else
		echo "Install failed! ${dirpath} is not empty. "
		exit 1
	fi
}
builddir $INSTALL_DIR
# get the path of the install program
INSTALLEXE_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
if [ -d ${INSTALLEXE_DIR} ];then
	cp ${INSTALLEXE_DIR}/* ${INSTALL_DIR} -r
else
	echo "Install failed! The path of the install program is not exist: ${INSTALLEXE_DIR}}"
	rm ${INSTALL_DIR} -r
	exit 1
fi
cd ${INSTALL_DIR}
INSTALL_DIR=$(pwd)
echo $INSTALL_DIR

# Completing empty folders that may be missing
## the root directory
if [ ! -d ${INSTALL_DIR}/input ];then
	mkdir -p ${INSTALL_DIR}/input
fi
if [ ! -d ${INSTALL_DIR}/cache/brdm ];then
	mkdir -p ${INSTALL_DIR}/cache/brdm
fi
if [ ! -d ${INSTALL_DIR}/cache/highrate ];then
	mkdir -p ${INSTALL_DIR}/cache/highrate
fi
if [ ! -d ${INSTALL_DIR}/cache/obspost ];then
	mkdir -p ${INSTALL_DIR}/cache/obspost
fi
if [ ! -d ${INSTALL_DIR}/work/work ];then
	mkdir -p ${INSTALL_DIR}/work/work
fi
## the template of update gnss.sit
if [ ! -d ${INSTALL_DIR}/template/rt_update_pos_bak/external_input ];then
	mkdir -p ${INSTALL_DIR}/template/rt_update_pos_bak/external_input
fi
if [ ! -d ${INSTALL_DIR}/template/rt_update_pos_bak/results ];then
	mkdir -p ${INSTALL_DIR}/template/rt_update_pos_bak/results
fi
if [ ! -d ${INSTALL_DIR}/template/rt_update_pos_bak/trace ];then
	mkdir -p ${INSTALL_DIR}/template/rt_update_pos_bak/trace
fi
if [ ! -d ${INSTALL_DIR}/template/rt_update_pos_bak/obs ];then
	mkdir -p ${INSTALL_DIR}/template/rt_update_pos_bak/obs
fi
## the template of nrtk
if [ ! -d ${INSTALL_DIR}/template/rt_nrtk_bak/external_input ];then
	mkdir -p ${INSTALL_DIR}/template/rt_nrtk_bak/external_input
fi
if [ ! -d ${INSTALL_DIR}/template/rt_nrtk_bak/results ];then
	mkdir -p ${INSTALL_DIR}/template/rt_nrtk_bak/results
fi
if [ ! -d ${INSTALL_DIR}/template/rt_nrtk_bak/trace ];then
	mkdir -p ${INSTALL_DIR}/template/rt_nrtk_bak/trace
fi
## the template of prepro
if [ ! -d ${INSTALL_DIR}/template/rt_prepro_smobs_bak/external_input ];then
	mkdir -p ${INSTALL_DIR}/template/rt_prepro_smobs_bak/external_input
fi
if [ ! -d ${INSTALL_DIR}/template/rt_prepro_smobs_bak/results ];then
	mkdir -p ${INSTALL_DIR}/template/rt_prepro_smobs_bak/results
fi
if [ ! -d ${INSTALL_DIR}/template/rt_prepro_smobs_bak/trace ];then
	mkdir -p ${INSTALL_DIR}/template/rt_prepro_smobs_bak/trace
fi
## the template of vrs
if [ ! -d ${INSTALL_DIR}/template/rt_vrs_bak/external_input ];then
	mkdir -p ${INSTALL_DIR}/template/rt_vrs_bak/external_input
fi
if [ ! -d ${INSTALL_DIR}/template/rt_vrs_bak/results ];then
	mkdir -p ${INSTALL_DIR}/template/rt_vrs_bak/results
fi
if [ ! -d ${INSTALL_DIR}/template/rt_vrs_bak/trace ];then
	mkdir -p ${INSTALL_DIR}/template/rt_vrs_bak/trace
fi
if [ ! -d ${INSTALL_DIR}/template/rt_vrs_bak/obs ];then
	mkdir -p ${INSTALL_DIR}/template/rt_vrs_bak/obs
fi

# check the bamboo install directory
if [ ! -d ${INSTALL_DIR}/bin/bbobin ];then
	mkdir -p ${INSTALL_DIR}/bin/bbobin
fi

# update the bamboo program
wget http://119.96.224.67:13019/data/cors_softwares/bamboo_release/bamboo -O ${INSTALL_DIR}/bin/bbobin/bamboo_new
if [ -s ${INSTALL_DIR}/bin/bbobin/bamboo_new ];then
	chmod 755 ${INSTALL_DIR}/bin/bbobin/bamboo_new
	mv ${INSTALL_DIR}/bin/bbobin/bamboo_new ${INSTALL_DIR}/bin/bbobin/bamboo
else
	echo "Install failed! Can not download the new version bamboo. "
	rm ${INSTALL_DIR} -r
	exit 1
fi

# set the global path
chmod +x ${INSTALL_DIR}/bbovrs_post.sh
chmod +x ${INSTALL_DIR}/bin/*

sed -i "s#^source globalvar.sh#source ${INSTALL_DIR}/globalvar.sh#g" ${INSTALL_DIR}/bbovrs_post.sh
sed -i "s#^source globalvar.sh#source ${INSTALL_DIR}/globalvar.sh#g" ${INSTALL_DIR}/bin/pro_smooth.sh
sed -i "s#^source globalvar.sh#source ${INSTALL_DIR}/globalvar.sh#g" ${INSTALL_DIR}/bin/decode.sh
sed -i "s#^source globalvar.sh#source ${INSTALL_DIR}/globalvar.sh#g" ${INSTALL_DIR}/bin/copy_allstation.sh
sed -i "s#^source globalvar.sh#source ${INSTALL_DIR}/globalvar.sh#g" ${INSTALL_DIR}/bin/down_allstation.sh
sed -i "s#^source globalvar.sh#source ${INSTALL_DIR}/globalvar.sh#g" ${INSTALL_DIR}/bin/judge_obsexists.sh
sed -i "s#^source globalvar.sh#source ${INSTALL_DIR}/globalvar.sh#g" ${INSTALL_DIR}/bin/sh_report.sh

sed -i "s#^rootdir=.*#rootdir=${INSTALL_DIR}#g" ${INSTALL_DIR}/globalvar.sh
sed -i "s#^bbobindir=.*#bbobindir=${INSTALL_DIR}/bin/bbobin#g" ${INSTALL_DIR}/globalvar.sh

echo "Install success."
